@@ -1,18 +1,19 @@
 import datetime
 from contextlib import contextmanager
 
-import operator
 from collections import OrderedDict
 from numbers import Number
+from os.path import join, dirname
 
-from AnyQt.QtWidgets import QLabel, QDateTimeEdit
-from AnyQt.QtCore import QDateTime, Qt, QSize, QTimer
+from AnyQt.QtWidgets import QLabel, QDateTimeEdit, QSizePolicy
+from AnyQt.QtCore import QDateTime, Qt, QSize, QTimer, QTimeZone
+from AnyQt.QtGui import QFontDatabase, QFont
 
 from Orange.data import Table, TimeVariable
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.widget import Input, Output
 
-from orangecontrib.timeseries import Timeseries
+from orangecontrib.timeseries import Timeseries, fromtimestamp, timestamp
 from orangecontrib.timeseries.util import add_time
 from orangecontrib.timeseries.widgets._rangeslider import ViolinSlider
 
@@ -28,40 +29,32 @@ class _TimeSliderMixin:
         self.__formatter = str  # type: Callable
 
     def setScale(self, minimum, maximum, time_delta):
-        self.__scale_minimum = datetime.datetime.fromtimestamp(
-            round(minimum),
-            tz=datetime.timezone.utc
-        )
-        self.__scale_maximum = datetime.datetime.fromtimestamp(
-            round(maximum),
-            tz=datetime.timezone.utc
-        )
+        self.__scale_minimum = fromtimestamp(round(minimum))
+        self.__scale_maximum = fromtimestamp(round(maximum))
         self.__time_delta = time_delta
 
     def scale(self, value):
         quantity = round(value - self.minimum())
         delta = self.__time_delta
         if delta is None:
-            return (self.__scale_minimum +
-                    (self.__scale_maximum - self.__scale_minimum) *
-                    (value - self.minimum()) /
-                    (self.maximum() - self.minimum())).timestamp()
+            return timestamp(
+                self.__scale_minimum
+                + (self.__scale_maximum - self.__scale_minimum)
+                * (value - self.minimum())
+                / (self.maximum() - self.minimum())
+            )
         scaled_dt = add_time(self.__scale_minimum, delta, quantity)
-        return scaled_dt.timestamp()
+        return timestamp(scaled_dt)
 
     def unscale(self, value):
         # Unscale e.g. absolute time to slider value
-        dt_val = datetime.datetime.fromtimestamp(round(value), tz=datetime.timezone.utc)
+        dt_val = fromtimestamp(round(value))
         delta = self.__time_delta
         if isinstance(delta, Number):
             diff = dt_val - self.__scale_minimum
             return round(diff / datetime.timedelta(milliseconds=1000 * delta))
         elif delta:
-            if delta[1] == 'day':
-                diff = dt_val - self.__scale_minimum
-                return self.minimum() + delta[0] \
-                       * diff.days
-            elif delta[1] == 'month':
+            if delta[1] == 'month':
                 return self.minimum() + delta[0] \
                        * ((dt_val.year - self.__scale_minimum.year) * 12
                           + max(dt_val.month - self.__scale_minimum.month,
@@ -98,6 +91,14 @@ def blockSignals(*objects):
         obj.blockSignals(False)
 
 
+def load_icons_font():
+    fontId = QFontDatabase.addApplicationFont(
+        join(dirname(__file__), 'icons', 'TimeSlice-icons.ttf')
+    )
+    font_family = QFontDatabase.applicationFontFamilies(fontId)[0]
+    return QFont(font_family, 9, QFont.Normal)
+
+
 class OWTimeSlice(widget.OWWidget):
     name = 'Time Slice'
     description = 'Select a slice of measurements on a time interval.'
@@ -116,7 +117,7 @@ class OWTimeSlice(widget.OWWidget):
 
     class Error(widget.OWWidget.Error):
         no_time_variable = widget.Msg('Data contains no time variable')
-        no_time_delta = widget.Msg('Data contains only 1 row')
+        no_time_delta = widget.Msg('Data contains only one time point')
 
     MAX_SLIDER_VALUE = 500
     DATE_FORMATS = ('yyyy', '-MM', '-dd', '  HH:mm:ss.zzz')
@@ -138,9 +139,9 @@ class OWTimeSlice(widget.OWWidget):
         ('3 hours', 10800),
         ('6 hours', 21600),
         ('12 hours', 43200),
-        ('1 day', (1, 'day')),
-        ('1 week', (7, 'day')),
-        ('2 weeks', (14, 'day')),
+        ('1 day', 86400),
+        ('1 week', 604800),
+        ('2 weeks', 1209600),
         ('1 month', (1, 'month')),
         ('2 months', (2, 'month')),
         ('3 months', (3, 'month')),
@@ -159,6 +160,8 @@ class OWTimeSlice(widget.OWWidget):
     playback_interval = settings.Setting(1)
     slider_values = settings.Setting((0, .2 * MAX_SLIDER_VALUE))
 
+    icons_font = None
+
     def __init__(self):
         super().__init__()
         self._delta = 0
@@ -172,11 +175,12 @@ class OWTimeSlice(widget.OWWidget):
                                       valuesChanged=self.sliderValuesChanged,
                                       minimumValue=self.slider_values[0],
                                       maximumValue=self.slider_values[1])
+        slider.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         slider.setShowText(False)
-        box = gui.vBox(self.controlArea, 'Time Slice')
-        box.layout().addWidget(slider)
+        selectBox = gui.vBox(self.controlArea, 'Select a Time Range')
+        selectBox.layout().addWidget(slider)
 
-        hbox = gui.hBox(box)
+        dtBox = gui.hBox(selectBox)
 
         kwargs = dict(calendarPopup=True,
                       displayFormat=' '.join(self.DATE_FORMATS),
@@ -206,47 +210,64 @@ class OWTimeSlice(widget.OWWidget):
             lambda: date_to.calendarWidget().repaint()
         )
 
-        hbox.layout().addStretch(100)
-        hbox.layout().addWidget(date_from)
-        hbox.layout().addWidget(QLabel(' – '))
-        hbox.layout().addWidget(date_to)
-        hbox.layout().addStretch(100)
+        dtBox.layout().addStretch(100)
+        dtBox.layout().addWidget(date_from)
+        dtBox.layout().addWidget(QLabel(' – '))
+        dtBox.layout().addWidget(date_to)
+        dtBox.layout().addStretch(100)
 
-        vbox = gui.vBox(self.controlArea, 'Step / Play Through')
-        gui.checkBox(vbox, self, 'loop_playback',
-                     label='Loop playback')
-        hbox = gui.hBox(vbox)
-        gui.checkBox(hbox, self, 'custom_step_size',
-                     label='Custom step size:',
+        hCenterBox = gui.hBox(self.controlArea)
+        gui.rubber(hCenterBox)
+        vControlsBox = gui.vBox(hCenterBox)
+
+        stepThroughBox = gui.vBox(vControlsBox, 'Step/Play Through',
+                                  sizePolicy=(QSizePolicy.Preferred,
+                                              QSizePolicy.Maximum))
+        gui.rubber(stepThroughBox)
+        gui.checkBox(stepThroughBox, self, 'loop_playback',
+                     label='Loop playback', attribute=Qt.WA_LayoutUsesWidgetRect)
+        customStepBox = gui.hBox(stepThroughBox)
+        gui.checkBox(customStepBox, self, 'custom_step_size',
+                     label='Custom step size: ',
                      toolTip='If not chosen, the active interval moves forward '
                              '(backward), stepping in increments of its own size.')
-        self.stepsize_combobox = gui.comboBox(hbox, self, 'step_size',
+        self.stepsize_combobox = gui.comboBox(customStepBox, self, 'step_size',
                                               items=tuple(self.STEP_SIZES.keys()),
                                               sendSelectedValue=True)
-        hbox = gui.hBox(vbox)
-        self.step_backward = gui.button(hbox, self, '⏮',
+        playBox = gui.hBox(stepThroughBox)
+        gui.rubber(playBox)
+        gui.rubber(stepThroughBox)
+
+        if self.icons_font is None:
+            self.icons_font = load_icons_font()
+
+        self.step_backward = gui.button(playBox, self, '⏪',
                                         callback=lambda: self.play_single_step(backward=True),
                                         autoDefault=False)
-        self.play_button = gui.button(hbox, self, '▶',
+        self.step_backward.setFont(self.icons_font)
+        self.play_button = gui.button(playBox, self, '▶️',
                                       callback=self.playthrough,
                                       toggleButton=True, default=True)
-        self.step_forward = gui.button(hbox, self, '⏭',
+        self.play_button.setFont(self.icons_font)
+        self.step_forward = gui.button(playBox, self, '⏩',
                                        callback=self.play_single_step,
                                        autoDefault=False)
+        self.step_forward.setFont(self.icons_font)
 
-        vbox = gui.vBox(self.controlArea, 'Playback/Tracking interval')
-        vbox.setToolTip('In milliseconds, set the delay for playback and '
-                        'for sending data upon manually moving the interval.')
+        gui.rubber(playBox)
+        intervalBox = gui.vBox(vControlsBox, 'Playback/Tracking interval')
+        intervalBox.setToolTip('In milliseconds, set the delay for playback and '
+                               'for sending data upon manually moving the interval.')
 
         def set_intervals():
             self.play_timer.setInterval(1000 * self.playback_interval)
             self.slider.tracking_timer.setInterval(1000 * self.playback_interval)
-        gui.valueSlider(vbox, self, 'playback_interval',
+        gui.valueSlider(intervalBox, self, 'playback_interval',
                         label='Delay:', labelFormat='%.2g sec',
                         values=self.DELAY_VALUES,
                         callback=set_intervals)
 
-        gui.rubber(self.controlArea)
+        gui.rubber(hCenterBox)
         self._set_disabled(True)
 
     def sliderValuesChanged(self, minValue, maxValue):
@@ -254,8 +275,8 @@ class OWTimeSlice(widget.OWWidget):
         minTime = self.slider.scale(minValue)
         maxTime = self.slider.scale(maxValue)
 
-        from_dt = QDateTime.fromMSecsSinceEpoch(minTime * 1000).toUTC()
-        to_dt = QDateTime.fromMSecsSinceEpoch(maxTime * 1000).toUTC()
+        from_dt = QDateTime.fromMSecsSinceEpoch(int(minTime * 1000)).toUTC()
+        to_dt = QDateTime.fromMSecsSinceEpoch(int(maxTime * 1000)).toUTC()
         if self.date_from.dateTime() != from_dt:
             with blockSignals(self.date_from):
                 self.date_from.setDateTime(from_dt)
@@ -272,7 +293,7 @@ class OWTimeSlice(widget.OWWidget):
             # maxValue's range is minValue's range shifted by one
             maxValue += 1
             maxTime = self.slider.scale(maxValue)
-            to_dt = QDateTime.fromMSecsSinceEpoch(maxTime * 1000).toUTC()
+            to_dt = QDateTime.fromMSecsSinceEpoch(int(maxTime * 1000)).toUTC()
             with blockSignals(self.date_to):
                 self.date_to.setDateTime(to_dt)
 
@@ -307,10 +328,10 @@ class OWTimeSlice(widget.OWWidget):
 
         if playing:
             self.play_timer.start()
-            self.play_button.setText('▮▮')
+            self.play_button.setText('⏸')
         else:
             self.play_timer.stop()
-            self.play_button.setText('▶')
+            self.play_button.setText('▶️')
 
         # hotfix
         self.repaint()
@@ -322,10 +343,9 @@ class OWTimeSlice(widget.OWWidget):
         def new_value(value):
             if self.custom_step_size:
                 step_amount = self.STEP_SIZES[self.step_size]
-                time = datetime.datetime.fromtimestamp(self.slider.scale(value),
-                                                       tz=datetime.timezone.utc)
+                time = fromtimestamp(self.slider.scale(value))
                 newTime = add_time(time, step_amount, -1 if backward else 1)
-                return self.slider.unscale(newTime.timestamp())
+                return self.slider.unscale(timestamp(newTime))
             return value + (-delta if backward else delta)
 
         if maxValue == self.slider.maximum() and not backward:
@@ -395,26 +415,29 @@ class OWTimeSlice(widget.OWWidget):
 
         time_values = data.time_values
 
-        min_dt = datetime.datetime.fromtimestamp(round(time_values.min()), tz=datetime.timezone.utc)
-        max_dt = datetime.datetime.fromtimestamp(round(time_values.max()), tz=datetime.timezone.utc)
+        min_dt = fromtimestamp(round(time_values.min()))
+        max_dt = fromtimestamp(round(time_values.max()))
 
         # Depending on time delta:
         #   - set slider maximum (granularity)
         #   - set range for end dt (+ 1 timedelta)
         #   - set date format
         #   - set time overlap options
-        delta = data.time_delta.min
+        delta = data.time_delta.gcd
         range = max_dt - min_dt
         if isinstance(delta, Number):
-            maximum = round(range / delta)
+            maximum = round(range.total_seconds() / delta)
 
             timedelta = datetime.timedelta(milliseconds=delta * 1000)
             min_dt2 = min_dt + timedelta
             max_dt2 = max_dt + timedelta
 
-            date_format = ''.join(self.DATE_FORMATS)
+            if delta >= 86400:  # more than a day
+                date_format = ''.join(self.DATE_FORMATS[0:3])
+            else:
+                date_format = ''.join(self.DATE_FORMATS)
 
-            for k, n in [n for n in self.STEP_SIZES.items()
+            for k, n in [(k, n) for k, n in self.STEP_SIZES.items()
                          if isinstance(n, Number)]:
                 if delta <= n:
                     min_overlap = k
@@ -422,23 +445,7 @@ class OWTimeSlice(widget.OWWidget):
             else:
                 min_overlap = '1 day'
         else:  # isinstance(delta, tuple)
-            if delta[1] == 'day':
-                maximum = range.days / delta[0]
-
-                timedelta = datetime.timedelta(days=delta[0])
-                min_dt2 = min_dt + timedelta
-                max_dt2 = max_dt + timedelta
-
-                date_format = ''.join(self.DATE_FORMATS[0:3])
-
-                for k, (i, u) in [(k, v) for k, v in self.STEP_SIZES.items()
-                                  if isinstance(v, tuple) and v[1] == 'day']:
-                    if delta[0] <= i:
-                        min_overlap = k
-                        break
-                else:
-                    min_overlap = '1 month'
-            elif delta[1] == 'month':
+            if delta[1] == 'month':
                 months = (max_dt.year - min_dt.year) * 12 + \
                          (max_dt.month - min_dt.month)
                 maximum = months / delta[0]
@@ -501,11 +508,7 @@ class OWTimeSlice(widget.OWWidget):
                     break
             else:
                 i, u = overlap
-                if u == 'day':
-                    if upper_overlap_limit <= datetime.timedelta(days=overlap[0]):
-                        max_overlap = k
-                        break
-                elif u == 'month':
+                if u == 'month':
                     month_diff = (max_dt.year - min_dt.year) * 12 \
                                  + max(0, max_dt.month - min_dt.month)
                     if month_diff / 2 <= i:
@@ -532,16 +535,21 @@ class OWTimeSlice(widget.OWWidget):
             self.stepsize_combobox.addItem(next_item)
 
         slider.setMinimum(0)
-        slider.setMaximum(maximum + 1)
+        slider.setMaximum(int(maximum + 1))
 
         self._set_disabled(False)
         slider.setHistogram(time_values)
         slider.setFormatter(var.repr_val)
-        slider.setScale(time_values.min(), time_values.max(), data.time_delta.min)
+        slider.setScale(time_values.min(), time_values.max(), data.time_delta.gcd)
         self.sliderValuesChanged(slider.minimumValue(), slider.maximumValue())
 
-        self.date_from.setDateTimeRange(min_dt, max_dt)
-        self.date_to.setDateTimeRange(min_dt2, max_dt2)
+        def utc_dt(dt):
+            qdt = QDateTime(dt)
+            qdt.setTimeZone(QTimeZone.utc())
+            return qdt
+
+        self.date_from.setDateTimeRange(utc_dt(min_dt), utc_dt(max_dt))
+        self.date_to.setDateTimeRange(utc_dt(min_dt2), utc_dt(max_dt2))
         self.date_from.setDisplayFormat(date_format)
         self.date_to.setDisplayFormat(date_format)
 
@@ -551,6 +559,9 @@ class OWTimeSlice(widget.OWWidget):
 
         self.slider.setFormatter(format_time)
 
+    def sizeHint(self):
+        return self.controlArea.sizeHint()
+
     @classmethod
     def migrate_settings(cls, settings_, version):
         if version < 2:
@@ -563,6 +574,7 @@ class OWTimeSlice(widget.OWWidget):
 
 if __name__ == '__main__':
     from AnyQt.QtWidgets import QApplication
+
     app = QApplication([])
     w = OWTimeSlice()
     data = Table('airpassengers')
